<?php
/*
 * Código para insertar una imagen de producto automaticamente
 * cuando se crea un producto nuevo
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Agregar campo personalizado para el metadato _orden_origen en la página de edición de productos
add_action('woocommerce_product_options_general_product_data', 'agregar_campo_metadato_orden_origen');

function agregar_campo_metadato_orden_origen() {
    woocommerce_wp_text_input(array(
        'id' => '_orden_origen',
        'label' => __('Orden Origen', 'woocommerce'),
        'placeholder' => __('Ingrese el número de orden origen', 'woocommerce'),
        'desc_tip' => 'true',
        'description' => __('Este número se usará para generar la imagen dinámica.', 'woocommerce')
    ));
}

// Guardar el valor del campo personalizado como metadato al guardar el producto
add_action('woocommerce_process_product_meta', 'guardar_metadato_orden_origen');

function guardar_metadato_orden_origen($product_id) {
_depu('*');
_depu('F1');
    $orden_origen = isset($_POST['_orden_origen']) ? sanitize_text_field($_POST['_orden_origen']) : '';
    update_post_meta($product_id, '_orden_origen', $orden_origen);

    // Crear y adjuntar la imagen al producto si el metadato ha cambiado
    if (!empty($orden_origen)) {
_depu('F2');
        crear_y_adjuntar_imagen_producto($product_id, $orden_origen);
    }
}

// Función para crear y adjuntar la imagen al producto
function crear_y_adjuntar_imagen_producto($product_id, $orden_origen) {
    // Verificar si la imagen ya ha sido creada previamente
    $ruta_imagen_existente = get_post_meta($product_id, '_imagen_producto', true);
_depu('F3');
    
    if (empty($ruta_imagen_existente)) {
_depu('CREANDO IMAGEN');
        // Crear la imagen solo si no existe
        $imagen = imagecreatetruecolor(300, 300);
        $color_blanco = imagecolorallocate($imagen, 255, 255, 255); // Color blanco
        imagefill($imagen, 0, 0, $color_blanco);

        // Agregar el número al centro de la imagen
        $fuente = 'arial.ttf'; // Puedes cambiar la fuente si lo deseas
        $color_texto = imagecolorallocate($imagen, 0, 0, 0); // Color negro
        $tamanio_fuente = 24; // Tamaño de la fuente
        $dimensiones_texto = imagettfbbox($tamanio_fuente, 0, $fuente, $orden_origen);
        $texto_ancho = $dimensiones_texto[4] - $dimensiones_texto[0];
        $texto_alto = $dimensiones_texto[1] - $dimensiones_texto[5];
        $x = (imagesx($imagen) - $texto_ancho) / 2;
        $y = (imagesy($imagen) - $texto_alto) / 2 + $texto_alto;
        imagettftext($imagen, $tamanio_fuente, 0, $x, $y, $color_texto, $fuente, $orden_origen);

        // Guardar la imagen en el directorio de subida de WordPress
        $nombre_imagen = 'imagen_producto_' . $product_id . '.png';
        $ruta_imagen = WP_CONTENT_DIR . '/uploads/' . $nombre_imagen;
        imagepng($imagen, $ruta_imagen);

        // Guardar la ruta de la imagen como metadato en el producto
        update_post_meta($product_id, '_imagen_producto', $ruta_imagen);

        // Liberar memoria
        imagedestroy($imagen);

        // Luego, adjuntar la imagen al producto como imagen destacada
        adjuntar_imagen_destacada_producto($product_id, $ruta_imagen);
    }
}

// Función para adjuntar la imagen como imagen destacada del producto
function adjuntar_imagen_destacada_producto($product_id, $ruta_imagen) {
    // Verificar si el archivo existe
_depu('ADJUNTAR');

    if (file_exists($ruta_imagen)) {
        // Adjuntar la imagen al producto
_depu('AGREGAR');
        $attachment_id = agregar_imagen_producto_wp($product_id, $ruta_imagen);
        
        if ($attachment_id) {
_depu('ASIGNAR');
            // Asignar la imagen adjunta como la imagen destacada del producto
            set_post_thumbnail($product_id, $attachment_id);
        }
    }
}

// Función para agregar la imagen al medio de WordPress
function agregar_imagen_producto_wp_old($product_id, $ruta_imagen) {
    // Agregar la imagen al medio de WordPress
_depu('AGREGANDO_F');

    $attachment_id = media_handle_sideload(array('file' => $ruta_imagen), $product_id);
    
    if (!is_wp_error($attachment_id)) {
		_depu('bien');
        return $attachment_id;
    } else {
		_depu('error');
        return false;
    }
}


// Función para agregar la imagen al medio de WordPress
function agregar_imagen_producto_wp($product_id, $ruta_imagen) {
_depu('AGREGANDO_F');
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    require_once(ABSPATH . 'wp-admin/includes/file.php');
    require_once(ABSPATH . 'wp-admin/includes/media.php');

    // Agregar la imagen al medio de WordPress
    $attachment_id = media_sideload_image($ruta_imagen, $product_id, '', 'id');
_depu($product_id);
    
    if (!is_wp_error($attachment_id)) {
_depu('bien');
        return $attachment_id;
    } else {
_depu($ruta_imagen);
        return false;
    }
}

