<?php
/*
 * Código para insertar una imagen de producto automaticamente
 * cuando se crea un producto nuevo
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Agregar nueva pestaña en la sección 'Product data' 
add_filter( 'woocommerce_product_data_tabs', 'custom_product_data_tab' );
function custom_product_data_tab( $tabs ) {
    $tabs['custom_tab'] = array(
        'label'    => __( 'Información Adicional', 'text-domain' ),
        'target'   => 'custom_tab_data',
        'class'    => array( 'show_if_simple', 'show_if_variable' ),
    );
    return $tabs;
}

// Agregar contenido a la pestaña
add_action( 'woocommerce_product_data_panels', 'custom_product_data_fields' );
function custom_product_data_fields() {
    ?>
    <div id='custom_tab_data' class='panel woocommerce_options_panel'>
        <div class='options_group'>
            <?php
            // Campo para Nro. de orden de origen
            woocommerce_wp_text_input( array(
                'id'          => 'origen_order_number',
                'label'       => __( 'Nro. de Orden de Origen', 'text-domain' ),
                'placeholder' => '',
                'desc_tip'    => 'true',
                'description' => __( 'Introduce el número de orden de origen.', 'text-domain' )
            ) );

            // Campo para Nombre Completo
            woocommerce_wp_text_input( array(
                'id'          => 'full_name',
                'label'       => __( 'Nombre Completo', 'text-domain' ),
                'placeholder' => '',
                'desc_tip'    => 'true',
                'description' => __( 'Introduce el nombre completo.', 'text-domain' )
            ) );

            // Campo para Monto total
			/*
            woocommerce_wp_text_input( array(
                'id'          => 'total_amount',
                'label'       => __( 'Monto Total', 'text-domain' ),
                'placeholder' => '',
                'desc_tip'    => 'true',
                'description' => __( 'Introduce el monto total.', 'text-domain' )
            ) );

            // Campo para Moneda (lista desplegable)
            woocommerce_wp_select( array(
                'id'          => 'currency',
                'label'       => __( 'Moneda', 'text-domain' ),
                'placeholder' => '',
                'desc_tip'    => 'true',
                'options'     => array(
                    'USD' => 'USD',
                    'EUR' => 'EUR',
                    'GBP' => 'GBP',
                    // Añade más monedas según sea necesario
                ),
                'description' => __( 'Selecciona la moneda.', 'text-domain' )
            ) );
			*/
            // Campo para Email
            woocommerce_wp_text_input( array(
                'id'          => 'email',
                'label'       => __( 'Email', 'text-domain' ),
                'placeholder' => '',
                'type'        => 'email',
                'desc_tip'    => 'true',
                'description' => __( 'Introduce la dirección de correo electrónico.', 'text-domain' )
            ) );

            // Campo para Teléfono
            woocommerce_wp_text_input( array(
                'id'          => 'phone',
                'label'       => __( 'Teléfono', 'text-domain' ),
                'placeholder' => '',
                'type'        => 'text',
                'desc_tip'    => 'true',
                'description' => __( 'Introduce el número de teléfono.', 'text-domain' )
            ) );

            // Campo para Dirección
            woocommerce_wp_textarea_input( array(
                'id'          => 'address',
                'label'       => __( 'Dirección', 'text-domain' ),
                'placeholder' => '',
                'desc_tip'    => 'true',
                'description' => __( 'Introduce la dirección.', 'text-domain' )
            ) );
            ?>
        </div>
    </div>
    <?php
}

// Guardar datos de los campos personalizados al guardar el producto
add_action( 'woocommerce_process_product_meta', 'save_custom_product_data_fields' );
function save_custom_product_data_fields( $post_id ) {
    // Guardar Nro. de orden de origen
    $origen_order_number = isset( $_POST['origen_order_number'] ) ? sanitize_text_field( $_POST['origen_order_number'] ) : '';
    update_post_meta( $post_id, 'origen_order_number', $origen_order_number );

    // Guardar Nombre Completo
    $full_name = isset( $_POST['full_name'] ) ? sanitize_text_field( $_POST['full_name'] ) : '';
    update_post_meta( $post_id, 'full_name', $full_name );

    // Guardar Monto total
    $total_amount = isset( $_POST['total_amount'] ) ? sanitize_text_field( $_POST['total_amount'] ) : '';
    update_post_meta( $post_id, 'total_amount', $total_amount );

    // Guardar Moneda
    $currency = isset( $_POST['currency'] ) ? sanitize_text_field( $_POST['currency'] ) : '';
    update_post_meta( $post_id, 'currency', $currency );

    // Guardar Email
    $email = isset( $_POST['email'] ) ? sanitize_email( $_POST['email'] ) : '';
    update_post_meta( $post_id, 'email', $email );

    // Guardar Teléfono
    $phone = isset( $_POST['phone'] ) ? sanitize_text_field( $_POST['phone'] ) : '';
    update_post_meta( $post_id, 'phone', $phone );

    // Guardar Dirección
    $address = isset( $_POST['address'] ) ? sanitize_textarea_field( $_POST['address'] ) : '';
    update_post_meta( $post_id, 'address', $address );
}



// Establecer todos los productos como virtuales al guardar
add_action( 'save_post_product', 'set_product_virtual_on_save', 10, 3 );
function set_product_virtual_on_save( $post_id, $post, $update ) {
_depu('*');
_depu('h');
_depu($post_id);
    global $wpdb;
    $q = "UPDATE {$wpdb->prefix}postmeta SET meta_value = 'yes' WHERE post_id = %d AND meta_key = '_virtual2'";
_depu($q);
    $result = $wpdb->query( $wpdb->prepare( $q, $post_id ) );
_depu($result);
}
