<?php
/*
 * Metodo de pago propio de WAO.
 * version 1.0
 * Fecha: 27 de marzo de 2024
 * Actualizado: 29 de marzo de 2024
 * Desarrollado por : Arturo
 * PayB
*/
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

/*
 * Código para PayB (Crear Pedido y Redirigir)
 */

add_action('rest_api_init', function () {
    register_rest_route('custom/v1', '/process-order', [
        'methods' => 'POST',
        'callback' => 'handle_order_from_shopa',
        'permission_callback' => '__return_true', // Ajusta la seguridad según sea necesario
    ]);
});

function handle_order_from_shopa(WP_REST_Request $request) {
    $params = $request->get_json_params();

    // Asegúrate de tener el ID del producto 'servicio1'
    $servicio1_id = 145744; // Ejemplo: ID de 'servicio1' en PayB

    // Crear una nueva orden en WooCommerce
    $order = wc_create_order();

	$product = wc_get_product($servicio1_id);
	// Crear un nuevo ítem de pedido para este producto
	$item = new WC_Order_Item_Product();
	$item->set_product($product);
	$item->set_quantity(1); // Cambia la cantidad si es necesario

	// Establecer el precio del ítem (podrías obtener estos valores de $params)
	$precio_sin_impuestos = $params['total']-$params['taxes'];
	$precio_con_impuestos = $params['total']; // El precio total (incluyendo impuestos) viene de $params
	$impuestos = $params['taxes']; // Total de impuestos para este ítem

	// Precio del ítem sin impuestos
	$item->set_subtotal($precio_sin_impuestos);
	$item->set_total($precio_sin_impuestos); // Puedes ajustar si hay descuentos

	// Asumiendo que tienes la cantidad total de impuestos, puedes asignarla directamente al ítem  
	$tax_array = array( 'total' => array( $impuestos ) ); // WooCommerce espera un array para los impuestos
	$item->set_taxes($tax_array);
	$template_items = '<ul style="border-left:solid 1px #5576c1;border-right:solid 1px#5576c1;margin-top:0px;padding-left:5px;">{lis}</ul>'; 	
	$lis = "";
	foreach($params['items'] as $itms){
		//$salida="".$itms['quantity']."".$itms['price']."";
		//$item->add_meta_data($itms['name'], $salida, true);
		$li_item = '<li>{quantity} - {name} - ({price})</li>';
		$li_item = str_replace('{name}',$itms['name'],$li_item);
		$li_item = str_replace('{quantity}',$itms['quantity'],$li_item);
		$li_item = str_replace('{price}',$itms['price'],$li_item);
		$lis.=$li_item;
	}
	$template_items = str_replace('{lis}',$lis,$template_items);

	update_option('_depu',$params['items']);
    //$order->update_meta_data('products', $template_items);

    $item->add_meta_data(___('Service for'), $template_items, true);
    //$item->add_meta_data('Service for:', '<b>Meta dato personal</b>', true);
	// Para asegurarte de que los impuestos se reflejan en el total del pedido, ajusta el total de impuestos del pedido
	$order->set_total($precio_con_impuestos);
	// Añadir el ítem configurado al pedido
	$order->add_item($item);

    // Añadir el producto 'servicio1' a la orden, usando el ID del producto y la cantidad
    //$order->add_product(wc_get_product($servicio1_id), 1); // Cambia la cantidad si es necesario

    // Ajustar los precios y taxes de la orden basándose en los datos recibidos
    //$order->set_total($params['total']); // Establece el total de la orden
    //$order->set_tax_total($params['taxes']); // Establece el total de la orden

    // Aplicar la información de facturación de la petición a la orden
    if (isset($params['billing_info'])) {
        foreach ($params['billing_info'] as $key => $value) {
            $setter = "set_billing_$key";
            if (is_callable([$order, $setter])) {
                $order->$setter($value);
            }
        }
    }

    // Guardar información adicional como metadatos del pedido
    $order->update_meta_data('shop_origin_url', $params['shop_url']);
    $order->update_meta_data('shop_origin_name', $params['shop_name']);
    $order->update_meta_data('payment_method_note', $params['payment_method_note']);
    //$order->update_meta_data('taxes1', $params['taxes']);

	// Crear el contenido de la nota
	$note_content = "Payment Method Note: " . $params['payment_method_note'] .
					"\nShop URL: " . $params['shop_url'] .
					"\nShop Name: " . $params['shop_name'] .
					"\nShopA Order ID: " . $params['order_id'];

	// Añadir la nota al pedido 
	$order->add_order_note($note_content, false); // El segundo parámetro a 'false' hace que la nota sea privada
	
	$order->calculate_totals(); 
	$order->save(); // Guardar los cambios hechos al pedido

    // Cambiar el estado de la orden a 'pending' (o cualquier otro estado inicial adecuado)
    $order->update_status('pending', 'Pedido creado automáticamente desde ShopA.');

    // Opcional: Redirigir al cliente a la página de pago de esta orden
    $checkout_url = $order->get_checkout_payment_url();

    return new WP_REST_Response(['checkout_url' => $checkout_url], 200);
}
// Insertar la informacion de billing en la pagina pay-order
function add_billing_info_to_order_pay_page() {
    // Comprobar si estamos en la página de pago de un pedido
    if (is_wc_endpoint_url('order-pay')) {
        $order_id = get_query_var('order-pay');
        $order = wc_get_order($order_id);
		$template='
		<h2>Billing information</h2>
		<table class="shop_table">
			<tr>
				<th width="25%">Name:</th>
				<td width="25%">{name}</td>
				<th width="25%">Direccion</th>
				<td width="25%">{address}</td>
			</tr>
			<tr>
				<th>Email</th>
				<td>{email}</td>
				<th>Telephone</th>
				<td>{phone}</td>
			</tr>
			<tr>
				<th>Country</th>
				<td>{country}</td>
				<th>City</th>
				<td>{city}</td>
			</tr>
		</table>
		';

        if ($order) {
            // Aquí puedes ajustar exactamente qué información quieres mostrar y cómo
			$template=str_replace('{name}',$order->get_billing_first_name().' '.$order->get_billing_last_name(),$template);
			$template=str_replace('{address}',$order->get_billing_address_1(),$template);
			$template=str_replace('{email}',$order->get_billing_email(),$template);
			$template=str_replace('{phone}',$order->get_billing_phone(),$template);
			$template=str_replace('{country}',$order->get_billing_country(),$template);
			$template=str_replace('{city}',$order->get_billing_city(),$template);
			echo $template;
        }
    }
}
//add_action('woocommerce_before_checkout_form', 'add_billing_info_to_order_pay_page', 20);//esto no funciona en ninguno
//add_action('woocommerce_pay_order_before_payment', 'add_billing_info_to_order_pay_page', 20);//esto funcionó en pagar.windowsandoffice.com
//add_action('woocommerce_pay_order_before_submit', 'add_billing_info_to_order_pay_page', 20);//esto muestra abajo

// Funcion para traducir pocas cadenas
if (!function_exists('___')) {
function ___($cadena) {
	$locale = get_locale();
	$idioma = substr($locale, 0, 2);
	// Arreglo de traducciones
    $traducciones = [
        'en' => [
            'To pay for this order, please contact %s' => 'To pay for this order, please contact %s',
			'You can pay your dispatched order by clicking <a href="%s">here</a>.' => 'You can pay your dispatched order by clicking <a href="%s">here</a>.',
			'You are paying with *Pay after invoice* method, that means you already have the product in your inbox email.'=>'You are paying with <b>Pay after invoice</b> method, that means you already have the product in your inbox email.',
			'Service for'=>'Service for',
            'Click <a href="%s" %s>here</a> to proceed to pay.' => 'Click <a href="%s" %s>here</a> to proceed to pay.',
            'You will receive the payment link in another email.' => 'You will receive the payment link in another email.',
			'Please, reenter your email.' => 'Please reenter your email*',
			'Email address' => 'Email address',
			'Name' => 'Full Name',
			'Address' => 'Address',
			'Email' => 'Email',
			'Phone' => 'Phone',
			'Country' => 'Country',
			'City' => 'City',
			'Billing Information' => '',
        ],
        'es' => [
            'To pay for this order, please contact %s' => 'Para pagar este pedido, por favor contacta a %s',
            'You can pay your dispatched order by clicking <a href="%s">here</a>.' => 'Puede pagar su pedido enviado haciendo clic <a href="%s">aquí</a>.',
			'You are paying with *Pay after invoice* method, that means you already have the product in your inbox email.'=>'Estás pagando con el método <b>Pago después de la factura</b>, eso significa que ya tienes el producto en tu bandeja de entrada de correo electrónico.',
			'Service for'=>'Servicio para',
            'Click <a href="%s" %s>here</a> to proceed to pay.' => 'Haga clic <a href="%s" %s>aquí</a> para proceder al pago.',
            'You will receive the payment link in another email.' => 'Recibirás el enlace de pago en otro correo electrónico.',
			'Please, reenter your email.' => 'Por favor, vuelve a introducir tu correo electrónico.',
			'Email address' => 'Direccion de correo electr&oacutenico',
			'Name' => 'Nombre',
			'Address' => 'Direcci&oacute;n',
			'Email' => 'Corr&eacute;o Electr&oacute;nico',
			'Phone' => 'Tel&eacute;fono',
			'Country' => 'Pa&iacute;s',
			'City' => 'Ciudad',
			'Billing Information' => 'Información de facturación',
        ],
        'it' => [
            'To pay for this order, please contact %s' => 'Per pagare questo ordine, si prega di contattare %s',
            'You can pay your dispatched order by clicking <a href="%s">here</a>.' => 'Puoi pagare il tuo ordine spedito facendo clic <a href="%s">qui</a>.',
			'You are paying with *Pay after invoice* method, that means you already have the product in your inbox email.'=>'Stai pagando con il metodo <b>Paga dopo la fattura</b>, ciò significa che hai già il prodotto nella tua casella di posta elettronica.',
			'Service for'=>'Servizio per',
            'Click <a href="%s" %s>here</a> to proceed to pay.' => 'Fai clic <a href="%s" %s>qui</a> per procedere con il pagamento.',
            'You will receive the payment link in another email.' => "Riceverai il link per il pagamento in un'altra email.",
			'Please, reenter your email.' => 'Per favore, inserisci nuovamente la tua email.',
			'Email address' => 'Indirizzo e-mail',
			'Name' => 'Nome e cognome',
			'Address' => 'Indirizzo',
			'Email' => 'E-mail',
			'Phone' => 'Telefono',
			'Country' => 'Paese',
			'City' => 'Città',
			'Billing Information' => 'Informazioni di fatturazione',
        ],
        'de' => [ 
            'To pay for this order, please contact %s' => 'Um diese Bestellung zu bezahlen, bitte kontaktieren Sie %s',
            'You can pay your dispatched order by clicking <a href="%s">here</a>.' => 'Sie können Ihre bereits versendete Bestellung bezahlen, indem Sie <a href="%s">hier</a> klicken.',
			'You are paying with *Pay after invoice* method, that means you already have the product in your inbox email.'=>'Sie zahlen mit der Methode <b>Zahlung nach Rechnung</b>, das heißt, Sie haben das Produkt bereits in Ihrem E-Mail-Posteingang.',
			'Service for'=>'Service für',
            'Click <a href="%s" %s>here</a> to proceed to pay.' => 'Klicken Sie <a href="%s" %s>hier</a>, um mit der Zahlung fortzufahren.',
            'You will receive the payment link in another email.' => 'Den Zahlungslink erhalten Sie in einer weiteren E-Mail.',
			'Please, reenter your email.' => 'Bitte geben Sie Ihre E-Mail-Adresse erneut ein.',
			'Email address' => 'E-Mail-Adresse',
			'Name' => 'Vollständiger Name',
			'Address' => 'Adresse',
			'Email' => 'Email',
			'Phone' => 'Telefon',
			'Country' => 'Land',
			'City' => 'Stadt',
			'Billing information' => 'Abrechnungsdaten',
        ],
        'fr' => [
            'To pay for this order, please contact %s' => 'Pour payer cette commande, veuillez contacter %s',
            'You can pay your dispatched order by clicking <a href="%s">here</a>.' => 'Vous pouvez payer votre commande expédiée en cliquant <a href="%s">ici</a>.',
			'You are paying with *Pay after invoice* method, that means you already have the product in your inbox email.'=>'Vous payez avec la méthode <b>Payer après facture</b>, cela signifie que vous avez déjà le produit dans votre boîte de réception.',
			'Service for'=>'Service pour',
            'Click <a href="%s" %s>here</a> to proceed to pay.' => 'Cliquez <a href="%s" %s>ici</a> pour procéder au paiement.',
            'You will receive the payment link in another email.' => 'Vous recevrez le lien de paiement dans un autre e-mail.',
			'Please, reenter your email.' => "S'il vous plaît, entrez à nouveau votre email.",
			'Email address' => 'Adresse e-mail',
			'Name' => 'Nom et prénom',
			'Address' => 'Adresse',
			'Email' => 'E-mail',
			'Phone' => 'Téléphone',
			'Country' => 'Pays',
			'City' => 'Ville',
			'Billing Information' => 'Informations de facturation',
        ],
        // Agrega más idiomas y traducciones según sea necesario
    ];

    // Verificar si el idioma y la cadena existen en el arreglo de traducciones
    if (isset($traducciones[$idioma][$cadena])) {
        return $traducciones[$idioma][$cadena];
    }

    // Si no se encuentra la traducción, devolver la cadena original
    return $cadena;
}

}

function _depu($masdato){
	$dato = get_option('_depu');
	if ($masdato == "*"){
		$dato = "";
	}else{
		$dato .= "|".$masdato;
	}
	update_option('_depu',$dato);
}