<?php
/**
 * Single Product Image
 *
 * This template can be overridden by copying it to yourtheme/woocommerce/single-product/product-image.php.
 *
 * HOWEVER, on occasion WooCommerce will need to update template files and you
 * (the theme developer) will need to copy the new files to your theme to
 * maintain compatibility. We try to do this as little as possible, but it does
 * happen. When this occurs the version of the template file will be bumped and
 * the readme will list any important changes.
 *
 * @see     https://woo.com/document/template-structure/
 * @package WooCommerce\Templates
 * @version 7.8.0
 */

defined( 'ABSPATH' ) || exit;

// Note: `wc_get_gallery_image_html` was added in WC 3.3.2 and did not exist prior. This check protects against theme overrides being used on older versions of WC.
if ( ! function_exists( 'wc_get_gallery_image_html' ) ) {
	return;
}

global $product;

$columns           = apply_filters( 'woocommerce_product_thumbnails_columns', 4 );
$post_thumbnail_id = $product->get_image_id();
$wrapper_classes   = apply_filters(
	'woocommerce_single_product_image_gallery_classes',
	array(
		'woocommerce-product-gallery',
		'woocommerce-product-gallery--' . ( $post_thumbnail_id ? 'with-images' : 'without-images' ),
		'woocommerce-product-gallery--columns-' . absint( $columns ),
		'images',
	)
);
?>
<div class="<?php echo esc_attr( implode( ' ', array_map( 'sanitize_html_class', $wrapper_classes ) ) ); ?>" data-columns="<?php echo esc_attr( $columns ); ?>" style="opacity: 0; transition: opacity .25s ease-in-out;">
	<div class="woocommerce-product-gallery__wrapper">
		<?php
		if ( $post_thumbnail_id ) {
			$html = wc_get_gallery_image_html( $post_thumbnail_id, true );
		} else {
			$html  = '<div class="woocommerce-product-gallery__image--placeholder">';
			$html .= sprintf( '<img src="%s" alt="%s" class="wp-post-image" />', esc_url( wc_placeholder_img_src( 'woocommerce_single' ) ), esc_html__( 'Awaiting product image', 'woocommerce' ) );
			$html .= '</div>';
		}

		echo apply_filters( 'woocommerce_single_product_image_thumbnail_html', $html, $post_thumbnail_id ); // phpcs:disable WordPress.XSS.EscapeOutput.OutputNotEscaped

		do_action( 'woocommerce_product_thumbnails' );
		?>
	</div>
</div>



<?php
// Obtener los metadatos del producto
$origen_order_number = get_post_meta( get_the_ID(), 'origen_order_number', true );
$full_name = get_post_meta( get_the_ID(), 'full_name', true );
$total_amount = get_post_meta( get_the_ID(), 'total_amount', true );
$currency = get_post_meta( get_the_ID(), 'currency', true );
$email = get_post_meta( get_the_ID(), 'email', true );
$phone = get_post_meta( get_the_ID(), 'phone', true );
$address = get_post_meta( get_the_ID(), 'address', true );

// Obtener el precio del producto
$product_price = get_post_meta( get_the_ID(), '_regular_price', true );

// Obtener la moneda del producto
$product_currency = get_woocommerce_currency_symbol();
?>

<div class="product-image-container" style="position:absolute"> 
    <div class="product-info-overlay">
        <p style="font-size:22px;font-weight: bold;text-align:center;"><?php echo esc_html( $origen_order_number ); ?></p>
        <p>Name: <?php echo esc_html( $full_name ); ?></p>
        <p>Total Amount: <?php echo ( $product_price ); ?> <?php echo esc_html( $product_currency ); ?></p>
        <p>Email: <?php echo esc_html( $email ); ?></p>
        <p>Phone: <?php echo esc_html( $phone ); ?></p>
        <p>Address: <?php echo esc_html( $address ); ?></p>
    </div>
</div>

<style>
    .product-image-container {
        position: relative;
        width: 300px;
        height: 300px;
    }
    .background-image {
        width: 100%;
        height: 100%;
    }
    .product-info-overlay {
        position: absolute;
        top: 0;
        left: 0;
        padding: 20px;
        background-color: rgba(255, 255, 255, 0.7);
		margin-top:30px;
		margin-left:20px;
		font-size:22px;
		width:300px;
    }
</style>
